

from django.db import models

from gm_serializer import fields, serializers


class UserManager(object):

    class Meta:
        db_column = "user_id"
        verbose_name = "User Magic Field"

    def __init__(self):
        self.url = 'api/user/get'

    def __call__(self, pk_list):
        # mock return user instance from rpc call.
        return [{'id': pk, 'username': "username %s" % pk} for pk in pk_list]


class Post(models.Model):

    """
    This is demonstrate app model.
    To explain how to use gm-serializer in your project.
    `user` is more like a cross-database foreign-key relation.
    """

    user = fields.MagicField(type=int, manager=UserManager, ttl=60*60)

    title = models.CharField(max_length=128)
    content = models.CharField(max_length=255)

    create_at = models.DateTimeField(auto_now_add=True)
    update_at = models.DateTimeField(auto_now=True)


class UniqueValidator(object):

    def __call__(self, *args, **kwargs):
        pass


class PostSerializer(serializers.MagicSerializer):

    user = serializers.MagicField(validators=[
        UniqueValidator
    ])

    class Meta:
        model = Post
        lazy_fields = ('user',)
        fields = ('user', 'title', 'content', 'create_at')

