
import json
import redis
from django.conf import settings

__all__ = []


class _PoolMixin(object):
    _pool = redis.ConnectionPool(**settings.GM_SERIALIZER['redis'])
    _client = redis.Redis(connection_pool=_pool)


class RedisCache(_PoolMixin):

    prefix = ''

    def __init__(self, prefix, ttl=3600):
        self.prefix = prefix
        self.ttl = ttl

    def __getitem__(self, item):
        """

        :param item:
        :return:
        """
        key = "%s:%s" % (self.prefix, str(item))
        obj = self._client.get(key)
        return json.loads(obj.decode()) if obj else None

    def __setitem__(self, key, value):
        """

        :param key:
        :param value:
        :return:
        """
        key = "%s:%s" % (self.prefix, str(key))
        if isinstance(value, dict):
            value = json.dumps(value)
        return self._client.set(key, value, self.ttl)



