
from gm_serializer.cache import RedisCache
from gm_serializer.logger import logger
from gm_serializer.utils import RPCMixin, DictWrapper


class CacheAccessor(RPCMixin):

    def __init__(self, name=None, manager=None, pk_list=None, ttl=None):
        """
        Cache accessor.
        :param name: field name, and cache prefix.
        :param manager: magic object manager.
        :param pk_list: magic object list.
        :param ttl: magic object redis cache ttl
        """
        self.name = name
        self.manager = manager() if type(manager) == type else manager
        self.pk_list = pk_list
        self.ttl = ttl

    @property
    def cache(self):
        return RedisCache(self.name, ttl=self.ttl)

    def get_object(self):
        """
        get object from cache.
        :return:
        """
        assert len(self.pk_list) == 1
        return self.cache[self.pk_list[0]]

    def get_queryset(self):
        """
        get list object from cache.
        :return:
        """
        values = list(filter(lambda i: i is not None, [self.cache[pk] for pk in set(self.pk_list)]))
        if len(values) == len(self.pk_list):
            return values
        return []


class MagicAccessor(CacheAccessor):

    def get_object(self):
        """

        :return:
        """
        obj = super(MagicAccessor, self).get_object()

        if obj:
            logger.debug("Magic Hit RedisCache: %s" % self.pk_list)
            return DictWrapper(obj)

        logger.debug("Magic CacheMiss: %s" % self.pk_list)
        try:
            obj = self.manager(pk_list=self.pk_list)[0]
        except IndexError:
            return None
        self.cache[obj['id']] = obj
        return DictWrapper(obj)

    def get_queryset(self):
        """

        :return:
        """
        queryset = super(MagicAccessor, self).get_queryset()
        if queryset:
            logger.debug("Magic Hit RedisCache: %s %s" % (self.name, self.pk_list))
            return [DictWrapper(obj) for obj in queryset]

        logger.debug("Magic CacheMiss: %s" % self.pk_list)
        data = self.manager(pk_list=self.pk_list)

        for obj in data:
            self.cache[obj['id']] = obj

        return [DictWrapper(d) for d in data]
