from gm_serializer.logger import logger
from gm_serializer.utils import DictWrapper


class MagicDescriptor(object):

    def __init__(self, field_with_rel):
        self.field = field_with_rel

    @property
    def cache_name(self):
        return "_%s_cache_" % self.field.attname

    @property
    def lazy_name(self):
        return "_%s_lazy_" % self.field.attname

    def __get__(self, instance, cls):
        """

        :param instance:
        :param cls:
        :return:
        """
        if instance is None:
            return self

        related_obj_pk = getattr(instance, "%s_id" % self.field.attname)

        if not related_obj_pk:
            # process `blank=True` and `null=True`
            return None

        if getattr(instance, self.lazy_name, False):
            return related_obj_pk

        try:
            lazy_obj = getattr(instance, self.cache_name)
            logger.debug("Magic Hit ObjCache: %s:%s" % (self.field.attname, related_obj_pk))
        except AttributeError:
            # update field pk_list, or you always get the same.
            self.field.accessor.pk_list = [related_obj_pk]
            lazy_obj = self.field.accessor.get_object()
            setattr(instance, self.cache_name, lazy_obj)
        return lazy_obj

    def __set__(self, instance, value):
        """

        :param instance:
        :param value:
        :return:
        """
        attname = self.field.attname
        accessor = self.field.accessor

        if value is None and self.field.null is False:
            raise ValueError("Null value not allowed.")

        if isinstance(value, DictWrapper):
            setattr(instance, self.cache_name, value)
            return

        setattr(accessor, "name", attname)
        setattr(accessor, "pk_list", [value])
        setattr(instance, "%s_id" % attname, value)

        if hasattr(instance, self.cache_name) and getattr(instance, self.cache_name)['id'] != value:
            obj_cache = self.field.manager.get_object()
            setattr(instance, "%s_id" % attname, value)
            setattr(instance, self.cache_name, obj_cache)


